package com.lokesh.sse.configuration;

import org.apache.kafka.clients.producer.ProducerConfig;
import org.apache.kafka.clients.producer.ProducerRecord;
import org.apache.kafka.common.serialization.StringSerializer;
import org.asynchttpclient.AsyncHttpClient;
import org.asynchttpclient.DefaultAsyncHttpClient;
import org.asynchttpclient.ListenableFuture;
import org.asynchttpclient.Response;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.autoconfigure.kafka.KafkaProperties;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import reactor.core.publisher.Flux;
import reactor.core.publisher.Mono;
import reactor.kafka.sender.KafkaSender;
import reactor.kafka.sender.SenderOptions;
import reactor.kafka.sender.SenderRecord;
import reactor.kafka.sender.SenderResult;

import java.io.IOException;
import java.text.SimpleDateFormat;
import java.util.HashMap;
import java.util.Map;
import java.util.Optional;
import java.util.concurrent.ExecutionException;

@Configuration
public class AsyncHttpClientConfig {
    private static final String TOPIC = "live-matches";
    private static final String BOOTSTRAP_SERVERS = "localhost:9092";
    private static final String CLIENT_ID_CONFIG = "news-consumer-rapidapi";
    @Autowired
    private KafkaSender<String, String> kafkaSender;

    public AsyncHttpClientConfig(String bootstrapServers){

        Map<String, Object> props = new HashMap<>();
        props.put(ProducerConfig.BOOTSTRAP_SERVERS_CONFIG, BOOTSTRAP_SERVERS);
        props.put(ProducerConfig.CLIENT_ID_CONFIG, CLIENT_ID_CONFIG);
        props.put(ProducerConfig.KEY_SERIALIZER_CLASS_CONFIG, StringSerializer.class.getName());
        props.put(ProducerConfig.VALUE_SERIALIZER_CLASS_CONFIG, StringSerializer.class.getName());

        SenderOptions<String, String> senderOptions = SenderOptions.create(props);
        kafkaSender = KafkaSender.create(senderOptions);
    }

    public void close() {
        kafkaSender.close();
    }
    @Bean
    public void sendNewsInKafka() throws IOException {
        AsyncHttpClient asyncHttpClient = new DefaultAsyncHttpClient();
        asyncHttpClient.prepare("GET", "https://livescore6.p.rapidapi.com/matches/v2/list-live?Category=soccer&Timezone=-7")
                .setHeader("X-RapidAPI-Key", "0b715b060amsh3056d92625990d9p1fe679jsn29ba530ad93c")
                .setHeader("X-RapidAPI-Host", "livescore6.p.rapidapi.com")
                .execute()
                .toCompletableFuture()
//                .thenAccept(System.out::println)
                .thenAccept(response -> {
                    String body = response.getResponseBody();
                    System.out.println(body);
                    ProducerRecord<String, String> record = new ProducerRecord<>(TOPIC, null, body);
                    SenderRecord<String, String, ProducerRecord<String, String>> senderRecord = SenderRecord.create(record, null);
                    kafkaSender.send(Flux.just(senderRecord))
                            .doOnNext(result -> {
                                if (result.exception() != null) {
                                    // Handle sending failure
                                    System.err.println("Failed to send message: " + result.exception().getMessage());
                                } else {
                                    // Handle sending success
                                    System.out.println("Message sent successfully: " + result.correlationMetadata());
                                }
                            })
                            .subscribe();
                })
                .thenAccept(System.out::println)
                .join();

        asyncHttpClient.close();
    }

    public static void main(String[] args) throws IOException {
        AsyncHttpClientConfig ob = new AsyncHttpClientConfig(BOOTSTRAP_SERVERS);
        ob.sendNewsInKafka();

    }
}
